package com.tzld.piaoquan.recommend.feature.service;

import com.google.common.reflect.TypeToken;
import com.tzld.piaoquan.recommend.feature.model.VideoFeature;
import com.tzld.piaoquan.recommend.feature.model.common.Result;
import com.tzld.piaoquan.recommend.feature.model.feature.GetVideoFeatureRequest;
import com.tzld.piaoquan.recommend.feature.model.feature.GetVideoFeatureResponse;
import com.tzld.piaoquan.recommend.feature.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;

/**
 * @author dyp
 */
@Service
@Slf4j
public class VideoFeatureService extends AbstractFeatureService<String, VideoFeature> {

    public VideoFeatureService() {
        super.emptyData = "{}";
        super.emptyDataExpire = 60;
        super.defaultExpire = 7200;
        super.keyFunc = k -> String.format("video:meta:%s", k);
        super.typeToken = new TypeToken<VideoFeature>() {
        };

        int maximumSize = 10000;
        int refreshAfterWrite = 60;
        int expireAfterWrite = 60;
        int expireAfterAccess = 60;
        initLocalCache(maximumSize, refreshAfterWrite, expireAfterWrite, expireAfterAccess);
    }


    public GetVideoFeatureResponse getVideoFeature(GetVideoFeatureRequest request) {
        String videoId = request.getVideoId();
        if (StringUtils.isBlank(videoId)) {
            return GetVideoFeatureResponse.newBuilder()
                    .setResult(Result.newBuilder().setCode(1))
                    .build();
        }
        VideoFeature videoFeature = super.get(videoId);
        return GetVideoFeatureResponse.newBuilder()
                .setResult(Result.newBuilder().setCode(1))
                .setVideoFeature(FeatureConverter.convert(videoFeature))
                .build();
    }

    public void saveVideoFeature(String jsonValue) {

        VideoFeature videoFeature = JSONUtils.fromJson(jsonValue, new TypeToken<VideoFeature>() {
        }, null);
        super.save(videoFeature.getVideoId(), videoFeature);
    }
}
